@@ -20,12 +20,15 @@ import getpass
 import logging
 import sys
 import traceback
+import typing as ty
 
+from cliff import _argparse
 from cliff import app
 from cliff import command
 from cliff import commandmanager
 from cliff import complete
 from cliff import help
+from cliff import interactive
 from oslo_utils import importutils
 from oslo_utils import strutils
 
@@ -45,7 +48,7 @@ DEFAULT_DOMAIN = 'default'
 DEFAULT_INTERFACE = 'public'
 
 
-def prompt_for_password(prompt=None):
+def prompt_for_password(prompt: str | None = None) -> str:
     """Prompt user for a password
 
     Prompt for a password if stdin is a tty.
@@ -63,36 +66,43 @@ def prompt_for_password(prompt=None):
             pass
     # No password because we did't have a tty or nothing was entered
     if not pw:
-        raise exc.CommandError(_("No password entered, or found via"
-                                 " --os-password or OS_PASSWORD"),)
+        raise exc.CommandError(
+            _(
+                "No password entered, or found via"
+                " --os-password or OS_PASSWORD"
+            ),
+        )
 
     return pw
 
 
 class OpenStackShell(app.App):
-
     CONSOLE_MESSAGE_FORMAT = '%(levelname)s: %(name)s %(message)s'
 
+    client_manager: clientmanager.ClientManager
+
     log = logging.getLogger(__name__)
-    timing_data = []
+    timing_data: list[ty.Any] = []
+    api_version: dict[str, str]
 
     def __init__(
-            self,
-            description=None,
-            version=None,
-            command_manager=None,
-            stdin=None,
-            stdout=None,
-            stderr=None,
-            interactive_app_factory=None,
-            deferred_help=False,
-    ):
+        self,
+        description: str | None = None,
+        version: str | None = None,
+        command_manager: commandmanager.CommandManager | None = None,
+        stdin: ty.TextIO | None = None,
+        stdout: ty.TextIO | None = None,
+        stderr: ty.TextIO | None = None,
+        interactive_app_factory: type['interactive.InteractiveApp']
+        | None = None,
+        deferred_help: bool = True,
+    ) -> None:
         # Patch command.Command to add a default auth_required = True
-        command.Command.auth_required = True
+        setattr(command.Command, 'auth_required', True)
 
         # Some commands do not need authentication
-        help.HelpCommand.auth_required = False
-        complete.CompleteCommand.auth_required = False
+        setattr(help.HelpCommand, 'auth_required', False)
+        setattr(complete.CompleteCommand, 'auth_required', False)
 
         # Slight change to the meaning of --debug
         self.DEFAULT_DEBUG_VALUE = None
@@ -104,34 +114,37 @@ class OpenStackShell(app.App):
         else:
             cm = command_manager
 
-        super(OpenStackShell, self).__init__(
-            description=__doc__.strip(),
+        super().__init__(
+            description=description or __doc__.strip(),
             version=version,
             command_manager=cm,
-            deferred_help=True,
+            stdin=stdin,
+            stdout=stdout,
+            stderr=stderr,
+            interactive_app_factory=interactive_app_factory,
+            deferred_help=deferred_help,
         )
 
         # Until we have command line arguments parsed, dump any stack traces
         self.dump_stack_trace = True
 
         # Set in subclasses
-        self.api_version = None
+        self.api_version = {}
 
-        self.client_manager = None
-        self.command_options = None
+        self.command_options: list[str] = []
 
-        self.do_profile = False
+        self.do_profile: bool = False
 
-    def configure_logging(self):
+    def configure_logging(self) -> None:
         """Configure logging for the app."""
         self.log_configurator = logs.LogConfigurator(self.options)
         self.dump_stack_trace = self.log_configurator.dump_trace
 
-    def run(self, argv):
+    def run(self, argv: list[str]) -> int:
         ret_val = 1
         self.command_options = argv
         try:
-            ret_val = super(OpenStackShell, self).run(argv)
+            ret_val = super().run(argv)
             return ret_val
         except Exception as e:
             if not logging.getLogger('').handlers:
@@ -144,15 +157,14 @@ class OpenStackShell(app.App):
             return ret_val
 
         finally:
-            if 'complete' not in argv:
-                self.log.info("END return value: %s", ret_val)
+            self.log.debug("END return value: %s", ret_val)
 
-    def init_profile(self):
+    def init_profile(self) -> None:
         self.do_profile = osprofiler_profiler and self.options.profile
         if self.do_profile:
             osprofiler_profiler.init(self.options.profile)
 
-    def close_profile(self):
+    def close_profile(self) -> None:
         if self.do_profile:
             profiler = osprofiler_profiler.get()
             trace_id = profiler.get_base_id()
@@ -163,32 +175,42 @@ class OpenStackShell(app.App):
             # printed. In fact we can define custom log level here with value
             # bigger than most big default one (CRITICAL) or something like
             # that (PROFILE = 60 for instance), but not sure we need it here.
-            self.log.warning("Trace ID: %s" % trace_id)
-            self.log.warning("Short trace ID "
-                             "for OpenTracing-based drivers: %s" % short_id)
-            self.log.warning("Display trace data with command:\n"
-                             "osprofiler trace show --html %s " % trace_id)
+            self.log.warning(f"Trace ID: {trace_id}")
+            self.log.warning(
+                f"Short trace ID for OpenTracing-based drivers: {short_id}"
+            )
+            self.log.warning(
+                "Display trace data with command:\n"
+                f"osprofiler trace show --html {trace_id} "
+            )
 
-    def run_subcommand(self, argv):
+    def run_subcommand(self, argv: list[str]) -> int:
         self.init_profile()
         try:
-            ret_value = super(OpenStackShell, self).run_subcommand(argv)
+            ret_value = super().run_subcommand(argv)
         finally:
             self.close_profile()
         return ret_value
 
-    def interact(self):
+    def interact(self) -> None:
         self.init_profile()
         try:
-            ret_value = super(OpenStackShell, self).interact()
+            ret_value = super().interact()
         finally:
             self.close_profile()
         return ret_value
 
-    def build_option_parser(self, description, version):
-        parser = super(OpenStackShell, self).build_option_parser(
+    def build_option_parser(
+        self,
+        description: str | None,
+        version: str | None,
+        argparse_kwargs: dict[str, ty.Any] | None = None,
+    ) -> _argparse.ArgumentParser:
+        parser = super().build_option_parser(
             description,
-            version)
+            version,
+            argparse_kwargs,
+        )
 
         # service token auth argument
         parser.add_argument(
@@ -244,11 +266,9 @@ class OpenStackShell(app.App):
             '--os-default-domain',
             metavar='<auth-domain>',
             dest='default_domain',
-            default=utils.env(
-                'OS_DEFAULT_DOMAIN',
-                default=DEFAULT_DOMAIN),
-            help=_('Default domain ID, default=%s. '
-                   '(Env: OS_DEFAULT_DOMAIN)') % DEFAULT_DOMAIN,
+            default=utils.env('OS_DEFAULT_DOMAIN', default=DEFAULT_DOMAIN),
+            help=_('Default domain ID, default=%s. (Env: OS_DEFAULT_DOMAIN)')
+            % DEFAULT_DOMAIN,
         )
         parser.add_argument(
             '--os-interface',
@@ -259,18 +279,23 @@ class OpenStackShell(app.App):
             # config key 'interface' will be ignored. Use OSC_Config's ctor
             # option 'override_defaults' below instead.
             default=utils.env('OS_INTERFACE'),
-            help=_('Select an interface type.'
-                   ' Valid interface types: [admin, public, internal].'
-                   ' default=%s, (Env: OS_INTERFACE)') % DEFAULT_INTERFACE,
+            help=_(
+                'Select an interface type.'
+                ' Valid interface types: [admin, public, internal].'
+                ' default=%s, (Env: OS_INTERFACE)'
+            )
+            % DEFAULT_INTERFACE,
         )
         parser.add_argument(
             '--os-service-provider',
             metavar='<service_provider>',
             dest='service_provider',
             default=utils.env('OS_SERVICE_PROVIDER'),
-            help=_('Authenticate with and perform the command on a service'
-                   ' provider using Keystone-to-keystone federation. Must'
-                   ' also specify the remote project option.')
+            help=_(
+                'Authenticate with and perform the command on a service'
+                ' provider using Keystone-to-keystone federation. Must'
+                ' also specify the remote project option.'
+            ),
         )
         remote_project_group = parser.add_mutually_exclusive_group()
         remote_project_group.add_argument(
@@ -278,16 +303,20 @@ class OpenStackShell(app.App):
             metavar='<remote_project_name>',
             dest='remote_project_name',
             default=utils.env('OS_REMOTE_PROJECT_NAME'),
-            help=_('Project name when authenticating to a service provider'
-                   ' if using Keystone-to-Keystone federation.')
+            help=_(
+                'Project name when authenticating to a service provider'
+                ' if using Keystone-to-Keystone federation.'
+            ),
         )
         remote_project_group.add_argument(
             '--os-remote-project-id',
             metavar='<remote_project_id>',
             dest='remote_project_id',
             default=utils.env('OS_REMOTE_PROJECT_ID'),
-            help=_('Project ID when authenticating to a service provider'
-                   ' if using Keystone-to-Keystone federation.')
+            help=_(
+                'Project ID when authenticating to a service provider'
+                ' if using Keystone-to-Keystone federation.'
+            ),
         )
         remote_project_domain_group = parser.add_mutually_exclusive_group()
         remote_project_domain_group.add_argument(
@@ -295,18 +324,22 @@ class OpenStackShell(app.App):
             metavar='<remote_project_domain_name>',
             dest='remote_project_domain_name',
             default=utils.env('OS_REMOTE_PROJECT_DOMAIN_NAME'),
-            help=_('Domain name of the project when authenticating to a'
-                   ' service provider if using Keystone-to-Keystone'
-                   ' federation.')
+            help=_(
+                'Domain name of the project when authenticating to a'
+                ' service provider if using Keystone-to-Keystone'
+                ' federation.'
+            ),
         )
         remote_project_domain_group.add_argument(
             '--os-remote-project-domain-id',
             metavar='<remote_project_domain_id>',
             dest='remote_project_domain_id',
             default=utils.env('OS_REMOTE_PROJECT_DOMAIN_ID'),
-            help=_('Domain ID of the project when authenticating to a'
-                   ' service provider if using Keystone-to-Keystone'
-                   ' federation.')
+            help=_(
+                'Domain ID of the project when authenticating to a'
+                ' service provider if using Keystone-to-Keystone'
+                ' federation.'
+            ),
         )
         parser.add_argument(
             '--timing',
@@ -331,7 +364,6 @@ class OpenStackShell(app.App):
             )
 
         return parser
-        # return clientmanager.build_plugin_option_parser(parser)
 
     """
     Break up initialize_app() so that overriding it in a subclass does not
@@ -346,11 +378,12 @@ class OpenStackShell(app.App):
     * ClientManager
 
     """
-    def _final_defaults(self):
+
+    def _final_defaults(self) -> None:
         # Set the default plugin to None
         # NOTE(dtroyer): This is here to set up for setting it to a default
         #                in the calling CLI
-        self._auth_type = None
+        self._auth_type: str | None = None
 
         # Converge project/tenant options
         project_id = getattr(self.options, 'project_id', None)
@@ -373,21 +406,21 @@ class OpenStackShell(app.App):
         # Save default domain
         self.default_domain = self.options.default_domain
 
-    def _load_plugins(self):
+    def _load_plugins(self) -> None:
         """Load plugins via stevedore
 
         osc-lib has no opinion on what plugins should be loaded
         """
         pass
 
-    def _load_commands(self):
+    def _load_commands(self) -> None:
         """Load commands via cliff/stevedore
 
         osc-lib has no opinion on what commands should be loaded
         """
         pass
 
-    def initialize_app(self, argv):
+    def initialize_app(self, argv: list[str]) -> None:
         """Global app init bits:
 
         * set up API versions
@@ -396,11 +429,18 @@ class OpenStackShell(app.App):
         """
 
         # Parent __init__ parses argv into self.options
-        super(OpenStackShell, self).initialize_app(argv)
-        self.log.info("START with options: %s",
-                      strutils.mask_password(" ".join(self.command_options)))
-        self.log.debug("options: %s",
-                       strutils.mask_password(self.options))
+        super().initialize_app(argv)
+        self.log.info(
+            "START with options: %s",
+            strutils.mask_password(" ".join(self.command_options))
+            if self.command_options
+            else "",
+        )
+        # https://review.opendev.org/c/openstack/oslo.utils/+/967979
+        self.log.debug(
+            "options: %s",
+            strutils.mask_password(self.options),  # type: ignore
+        )
 
         # Callout for stuff between superclass init and o-c-c
         self._final_defaults()
@@ -414,7 +454,7 @@ class OpenStackShell(app.App):
                     'auth_type': self._auth_type,
                 },
             )
-        except (IOError, OSError) as e:
+        except OSError as e:
             self.log.critical("Could not read clouds.yaml configuration file")
             self.print_help_if_requested()
             raise e
@@ -435,11 +475,11 @@ class OpenStackShell(app.App):
         self.log_configurator.configure(self.cloud)
         self.dump_stack_trace = self.log_configurator.dump_trace
         self.log.debug("defaults: %s", self.cloud_config.defaults)
-        self.log.debug("cloud cfg: %s",
-                       strutils.mask_password(self.cloud.config))
-
-        # Callout for stuff between o-c-c and ClientManager
-        # self._initialize_app_2(self.options)
+        # https://review.opendev.org/c/openstack/oslo.utils/+/967979
+        self.log.debug(
+            "cloud cfg: %s",
+            strutils.mask_password(self.cloud.config),  # type: ignore
+        )
 
         self._load_plugins()
 
@@ -454,23 +494,23 @@ class OpenStackShell(app.App):
             pw_func=prompt_for_password,
         )
 
-    def prepare_to_run_command(self, cmd):
+    def prepare_to_run_command(self, cmd: 'command.Command') -> None:
         """Set up auth and API versions"""
-        self.log.info(
+        self.log.debug(
             'command: %s -> %s.%s (auth=%s)',
             getattr(cmd, 'cmd_name', '<none>'),
             cmd.__class__.__module__,
             cmd.__class__.__name__,
-            cmd.auth_required,
+            getattr(cmd, 'auth_required', None),
         )
 
         # NOTE(dtroyer): If auth is not required for a command, skip
         #                get_one()'s validation to avoid loading plugins
-        validate = cmd.auth_required
+        validate = getattr(cmd, 'auth_required', False)
 
         # NOTE(dtroyer): Save the auth required state of the _current_ command
         #                in the ClientManager
-        self.client_manager._auth_required = cmd.auth_required
+        self.client_manager._auth_required = validate
 
         # Validate auth options
         self.cloud = self.cloud_config.get_one(
@@ -484,25 +524,31 @@ class OpenStackShell(app.App):
         # Push the updated args into ClientManager
         self.client_manager._cli_options = self.cloud
 
-        if cmd.auth_required:
+        if validate:
             self.client_manager.setup_auth()
             if hasattr(cmd, 'required_scope') and cmd.required_scope:
                 # let the command decide whether we need a scoped token
                 self.client_manager.validate_scope()
             # Trigger the Identity client to initialize
-            self.client_manager.session.auth.auth_ref = \
+            self.client_manager.session.auth.auth_ref = (  # type: ignore
                 self.client_manager.auth_ref
+            )
         return
 
-    def clean_up(self, cmd, result, err):
+    def clean_up(
+        self,
+        cmd: 'command.Command',
+        result: int,
+        err: BaseException | None,
+    ) -> None:
         self.log.debug('clean_up %s: %s', cmd.__class__.__name__, err or '')
 
         # Close SDK connection if available to have proper cleanup there
-        if hasattr(self.client_manager, "sdk_connection"):
+        if getattr(self.client_manager, "sdk_connection", None) is not None:
             self.client_manager.sdk_connection.close()
 
         # Close session if available
-        if hasattr(self.client_manager.session, "session"):
+        if getattr(self.client_manager, "session", None) is not None:
             self.client_manager.session.session.close()
 
         # Process collected timing data
@@ -519,8 +565,11 @@ class OpenStackShell(app.App):
             # If anything other than prettytable is specified, force csv
             format = 'table'
             # Check the formatter used in the actual command
-            if hasattr(cmd, 'formatter') \
-                    and cmd.formatter != cmd._formatter_plugins['table'].obj:
+            if (
+                hasattr(cmd, 'formatter')
+                and hasattr(cmd, '_formatter_plugins')
+                and cmd.formatter != cmd._formatter_plugins['table'].obj
+            ):
                 format = 'csv'
 
             sys.stdout.write('\n')
@@ -528,7 +577,7 @@ class OpenStackShell(app.App):
             tcmd.run(targs)
 
 
-def main(argv=None):
+def main(argv: list[str] | None = None) -> int:
     if argv is None:
         argv = sys.argv[1:]
     return OpenStackShell().run(argv)
